package com.tianyi.service.impl;


import com.tianyi.model.RequestModel;
import com.tianyi.service.PdfToOfdService;
import lombok.SneakyThrows;
import org.apache.commons.lang3.StringUtils;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.pdmodel.common.PDRectangle;
import org.apache.pdfbox.rendering.PDFRenderer;
import org.ofdrw.graphics2d.OFDGraphicsDocument;
import org.ofdrw.graphics2d.OFDPageGraphics2D;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.net.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;

@Service
public class PdfToOfdServiceImpl implements PdfToOfdService {

    private static final Logger log = LoggerFactory.getLogger(PdfToOfdServiceImpl.class);

    @Override
    @SneakyThrows
    public Object convert(RequestModel requestModel) {

        URL url = new URL(requestModel.getUrlStr());
        URLConnection urlConnection = url.openConnection();
        InputStream inputStream = urlConnection.getInputStream();
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        try (OFDGraphicsDocument ofdDoc = new OFDGraphicsDocument(outputStream); PDDocument pdfDoc = PDDocument.load(inputStream)) {
            PDFRenderer pdfRender = new PDFRenderer(pdfDoc);
            for (int pageIndex = 0; pageIndex < pdfDoc.getNumberOfPages(); pageIndex++) {
                PDRectangle pdfPageSize = pdfDoc.getPage(pageIndex).getBBox();
                OFDPageGraphics2D ofdPageG2d = ofdDoc.newPage(pdfPageSize.getWidth(), pdfPageSize.getHeight());
                pdfRender.renderPageToGraphics(pageIndex, ofdPageG2d);
            }
        }
        Path out;
        String os = System.getProperty("os.name").toLowerCase();
        log.error("系统类别:{}", os);
        if (StringUtils.contains(os, "linux")) {
            out = Paths.get("/home/ofd/ofddir/" + System.currentTimeMillis() + ".ofd");
            Files.write(out, outputStream.toByteArray());
            Path absolutePath = out.toAbsolutePath();
            return convertToHttpUrl(absolutePath.toString(),"http",8848,"/ofd-service");
        }
        if (StringUtils.contains(os, "windows")) {
            out = Paths.get("target", System.currentTimeMillis() + ".ofd");
            Files.write(out, outputStream.toByteArray());
            URI uri = out.toUri();
            return uri.getPath();
        }
        return null;
    }

    @SneakyThrows
    public static String convertToHttpUrl(String absolutePath, String scheme, int serverPort, String contextPath) {
        // 将绝对路径转换为URI对象
        URI uri = URI.create(absolutePath);

        // 构建HTTP URL
        StringBuilder httpUrl = new StringBuilder();

        httpUrl.append(scheme).append("://")
                .append("192.168.111.229").append(":")
                .append(serverPort).append(contextPath);

        // 如果URI中包含路径，则添加到HTTP URL中
        String path = uri.getPath();
        if (path != null && !path.isEmpty()) {
            httpUrl.append(path.startsWith("/") ? path : "/".concat(path));
        }

        // 如果URI中包含查询参数，则添加到HTTP URL中
        String query = uri.getQuery();
        if (query != null) {
            httpUrl.append("?").append(query);
        }

        // 如果URI中包含片段标识符，则添加到HTTP URL中
        String fragment = uri.getFragment();
        if (fragment != null) {
            httpUrl.append("#").append(fragment);
        }

        return httpUrl.toString();
    }
}
